/**
 * Title :        XLogo
 * Description :  XLogo is an interpreter for the Logo 
 * 						programming language
 * @author Loc Le Coq
 */

package xlogo;
import java.awt.*;
import java.io.*;

import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import javax.swing.text.rtf.RTFEditorKit;

import xlogo.utils.WebPage;
import java.util.Locale;
import xlogo.utils.Utils;
import xlogo.utils.ExtensionFichier;
import xlogo.kernel.Workspace;
import xlogo.kernel.Procedure;
import xlogo.kernel.Kernel;
public class MenuListener extends JDialog implements ActionListener{
	protected static final String FILE_NEW="new";
	protected static final String FILE_OPEN="open";
	protected static final String FILE_SAVE_AS="save_as";
	protected static final String FILE_SAVE="save";
	protected static final String FILE_SAVE_IMAGE="record_image";
	protected static final String FILE_COPY_IMAGE="copy_image";
	protected static final String FILE_PRINT_IMAGE="print_image";
	protected static final String FILE_SAVE_TEXT="save_text";
	protected static final String FILE_QUIT="quit";
	protected static final String EDIT_SELECT_ALL="select_all";
	protected static final String EDIT_COPY="copy";
	protected static final String EDIT_CUT="cut";
	protected static final String EDIT_PASTE="paste";
	protected static final String TOOLS_PEN_COLOR="pen_color";
	protected static final String TOOLS_SCREEN_COLOR="screen_color";
	protected static final String TOOLS_TRANSLATOR="translator";
	protected static final String TOOLS_OPTIONS="preferences";
	protected static final String TOOLS_START_FILE="start_file";
	protected static final String TOOLS_ERASER="eraser";
	protected static final String HELP_LICENCE="licence";
	protected static final String HELP_TRANSLATED_LICENCE="translated_licence";
	protected static final String HELP_ABOUT="about";
	private static final String WEB_SITE="http://xlogo.tuxfamily.org";
	private static final String MAIL="loic@xlogo.tuxfamily.org";
	private Application cadre;
	private Workspace wp;
	private Kernel kernel;
  
  public MenuListener() {
  }
  public MenuListener(Application cadre){
    this.cadre=cadre;
    this.kernel=cadre.getKernel();
    this.wp=kernel.getWorkspace();
    
  }
  public void actionPerformed(ActionEvent e){
    String cmd=e.getActionCommand();
    if (MenuListener.FILE_QUIT.equals(cmd)){     //Quitter
    	cadre.closeWindow();
    }
    else if (MenuListener.EDIT_COPY.equals(cmd)){   //Copier
      cadre.copy();
    }
    else if (MenuListener.EDIT_CUT.equals(cmd)){   //Couper
      cadre.cut();
    }
    else if (MenuListener.EDIT_PASTE.equals(cmd)){   //Coller
      cadre.paste();
    }
    else if (MenuListener.EDIT_SELECT_ALL.equals(cmd)){ // select all
    	cadre.select_all();
    }
    else if(MenuListener.FILE_NEW.equals(cmd)){ //nouveau
		String[] choix={Logo.messages.getString("ok"),Logo.messages.getString("annuler")};
		Object message=Logo.messages.getString("enregistrer_espace");
		ImageIcon icone=new ImageIcon(Utils.class.getResource("icone.png"));
		int retval=JOptionPane.showOptionDialog(cadre,message,Logo.messages.getString("enregistrer_espace"),JOptionPane.OK_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE,icone,choix,choix[0]);	
		if (retval==JOptionPane.OK_OPTION){
			wp.deleteAllProcedures();
			wp.deleteAllVariables();
			if (null!=Application.path) {
				Application.path=null;
				cadre.setTitle("XLOGO");
			}
			cadre.setEnabled_New(false);	
		  }
		  else cadre.setEnabled_New(true);
    }
    else if(MenuListener.FILE_SAVE.equals(cmd)|MenuListener.FILE_SAVE_AS.equals(cmd)){   //Enregistrer
      String path=Application.path;
      if (MenuListener.FILE_SAVE_AS.equals(cmd)||null==path) {
        JFileChooser jf=new JFileChooser(Utils.SortieTexte(Config.defaultFolder));
        jf.addChoosableFileFilter(new ExtensionFichier(Logo.messages.getString("fichiers_logo"),".lgo"));
				Utils.recursivelySetFonts(jf,Config.police);
        int retval=jf.showDialog(cadre,Logo.messages.getString("enregistrer"));
        if (retval==JFileChooser.APPROVE_OPTION){
          path=jf.getSelectedFile().getPath();
          String path2=path.toLowerCase();  // on garde la casse du path pour les systmes d'exploitation faisant la diffrence
          if (!path2.endsWith(".lgo")) path+=".lgo";
          Application.path=path;
					cadre.setEnabled_Record(true);
					cadre.setTitle("XLOGO"+"          "+path);
					try {
						File f=new File(path);
						Config.defaultFolder=Utils.rajoute_backslash(f.getParent());
					}
					catch(NullPointerException e2){}
        }
      }
      try{
        String aecrire="";
        try {
          for (int i = 0; i < wp.getNumberOfProcedure(); i++) {
            Procedure procedure = wp.getProcedure(i);
            if (procedure.affichable){
              aecrire += Logo.messages.getString("pour") + " " + procedure.name;
              for (int j = 0; j < procedure.nbparametre; j++) {
                aecrire += " :" + procedure.variable.get(j).toString();
              }
              aecrire += "\n" + procedure.instruction + Logo.messages.getString("fin") + "\n\n";
            }
          }
        }catch(NullPointerException ex){ex.printStackTrace();} //Si aucune procdure n'a t dfinie.
        Utils.writeLogoFile(path,aecrire);
      }
      catch(NullPointerException e3){System.out.println("annulation");} //Si l'utilisateur annule
      catch(IOException e2){cadre.ecris("erreur",Logo.messages.getString("ioecriture"));}
		}
    else if(MenuListener.FILE_OPEN.equals(cmd)){              //Ouvrir
      JFileChooser jf=new JFileChooser(Utils.SortieTexte(Config.defaultFolder));
      jf.addChoosableFileFilter(new ExtensionFichier(Logo.messages.getString("fichiers_logo"),".lgo"));
			Utils.recursivelySetFonts(jf,Config.police);
      int retval=jf.showDialog(cadre,Logo.messages.getString("ouvrir"));
      if (retval==JFileChooser.APPROVE_OPTION){
        String txt="";
        String path=jf.getSelectedFile().getPath();
        try{
        	txt=Utils.readLogoFile(path);
        }
        catch(IOException e1){cadre.ecris("erreur",Logo.messages.getString("iolecture"));}
        if (!txt.equals("")){
          if (!cadre.editeur.isVisible()) cadre.editeur.setVisible(true);
         	 cadre.editeur.setEditorStyledText(txt);
        }
       cadre.tmp_path=path;
			}
    }
    else if(MenuListener.TOOLS_SCREEN_COLOR.equals(cmd)){      //couleur du fond
    Color color=JColorChooser.showDialog(cadre,Logo.messages.getString("couleur_du_fond"),kernel.getScreenBackground());
    if (null!=color){
      Locale locale=Logo.generateLocale(Config.langage);
      java.util.ResourceBundle rs=java.util.ResourceBundle.getBundle("primitives",locale);
      String fcfg=rs.getString("fcfg");
      fcfg=fcfg.substring(0,fcfg.indexOf(" "));
      cadre.ecris("commentaire",fcfg+" ["+color.getRed()+" "+color.getGreen()+" "+color.getBlue()+"]\n");
      kernel.fcfg(color);
/*      Graphics g=Ardoise.dessin.getGraphics();
      g.setColor(color);
      g.fillRect(0,0,Config.imageWidth,Config.imageHeight);
      kernel.dg.setScreenBackground(color);
      kernel.dg.videecran();
      Graphics graph=cadre.getArdoise().getGraphics();
      graph.setClip((int)cadre.getActiveTurtle().corX-26,(int)cadre.getActiveTurtle().corY-26,52,52);
      graph.drawImage(Ardoise.dessin,0,0,cadre.getArdoise());
      cadre.getArdoise().repaint();*/
    }
   }
   else if(MenuListener.TOOLS_PEN_COLOR.equals(cmd)){      //Couleur du crayon
    Color color=JColorChooser.showDialog(cadre,Logo.messages.getString("couleur_du_crayon"),kernel.getActiveTurtle().couleurcrayon);
    if (null!=color){
      Locale locale=Logo.generateLocale(Config.langage);
      java.util.ResourceBundle rs=java.util.ResourceBundle.getBundle("primitives",locale);
      String fcc=rs.getString("fcc");
      fcc=fcc.substring(0,fcc.indexOf(" "));
      cadre.ecris("commentaire",fcc+" ["+color.getRed()+" "+color.getGreen()+" "+color.getBlue()+"]\n");
      kernel.fcc(color);
    }
  }
  else if(MenuListener.TOOLS_OPTIONS.equals(cmd)){  //preferences
			if (cadre.editeur.isVisible()) JOptionPane.showMessageDialog(cadre,Logo.messages.getString("ferme_editeur"),Logo.messages.getString("erreur"),JOptionPane.ERROR_MESSAGE);
			else cadre.prefOpen();
  }
	else if(MenuListener.TOOLS_TRANSLATOR.equals(cmd)){
		cadre.traducOpen();
	}
	else if (MenuListener.FILE_COPY_IMAGE.equals(cmd)){  // Copier l'image au presse-papier
		cadre.diagOpen(Logo.messages.getString("copier_image"));
  }
  else if (MenuListener.FILE_SAVE_IMAGE.equals(cmd)){  //Enregistrer l'image au format png ou jpg
  	cadre.diagOpen(Logo.messages.getString("enregistrer"));
  }
  else if (MenuListener.FILE_PRINT_IMAGE.equals(cmd)){    //imprimer l'image
  	cadre.diagOpen(Logo.messages.getString("imprimer"));
  }
  else if (MenuListener.FILE_SAVE_TEXT.equals(cmd)){
  	RTFEditorKit myRTFEditorKit = new RTFEditorKit();
   	StyledDocument myStyledDocument =cadre.getHistoryPanel().sd_Historique();
   	try{
   		JFileChooser jf=new JFileChooser(Utils.SortieTexte(Config.defaultFolder));
  	    jf.addChoosableFileFilter(new ExtensionFichier(Logo.messages.getString("fichiers_rtf"),".rtf"));
  	    Utils.recursivelySetFonts(jf,Config.police);
  	    int retval=jf.showDialog(cadre,Logo.messages.getString("enregistrer"));
  	    if (retval==JFileChooser.APPROVE_OPTION){
  	    	String path=jf.getSelectedFile().getPath();
  	        String path2=path.toLowerCase();  // on garde la casse du path pour les systmes d'exploitation faisant la diffrence
  	        if (!path2.endsWith(".rtf")) path+=".rtf";
  	        	FileOutputStream myFileOutputStream = new FileOutputStream(path);
	  	    	myRTFEditorKit.write(myFileOutputStream, myStyledDocument, 0,myStyledDocument.getLength()-1);
	  	    	myFileOutputStream.close();
  	         }
  	    }
  	    catch(FileNotFoundException e1){}
  	    catch(IOException e2){}
  	    catch(BadLocationException e3){}
  	    catch(NullPointerException e4){}
  }
  else if (MenuListener.HELP_ABOUT.equals(cmd)){   //Bote de dialogue  props
  	String message=Logo.messages.getString("message_a_propos1")+Config.version+"\n\n"
		+Logo.messages.getString("message_a_propos2")+" "+MenuListener.WEB_SITE+"\n\n"+Logo.messages.getString("message_a_propos3")+"\n     "+MenuListener.MAIL;
		JTextArea jt=new JTextArea(message);
		jt.setEditable(false);
		jt.setBackground(new Color(255,255,177));
		jt.setFont(Config.police);
		ImageIcon icone=new ImageIcon(Utils.class.getResource("icone.png"));
		JOptionPane.showMessageDialog(cadre,jt,Logo.messages.getString("a_propos"),JOptionPane.INFORMATION_MESSAGE,(Icon)icone);
  }
  else if (MenuListener.HELP_LICENCE.equals(cmd)|MenuListener.HELP_TRANSLATED_LICENCE.equals(cmd)){     // Affichage de la licence
   JFrame frame=new JFrame("Licence");
	frame.setIconImage(Toolkit.getDefaultToolkit().createImage(
			WebPage.class.getResource("icone.png")));
    frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    frame.setSize(500,500);
    WebPage editorPane = new WebPage();
    editorPane.setEditable(false);
    String path="gpl/gpl-";
    if (MenuListener.HELP_LICENCE.equals(cmd))  path+="en";
    else {
    	switch(Config.langage){
    		case Config.LANGUAGE_FRENCH: //french
    			path+="fr";
    		break;
    		case Config.LANGUAGE_ENGLISH: // english
    			path+="en";
    		break;
    		case Config.LANGUAGE_ARABIC: // Arabic
    			path+="ar";
    		break;
    		case Config.LANGUAGE_SPANISH: // spanish
    			path+="sp";
    		break;
    		case Config.LANGUAGE_PORTUGAL: // portuguese
    			path+="pt";	
    		break;
    		case Config.LANGUAGE_ESPERANTO: //esperanto
    			path+="eo";
    		break;
    		case Config.LANGUAGE_GERMAN: // german
    			path+="de";
    		break;
    		case Config.LANGUAGE_DUTCH: // holland
    			path+="nl";
    		break;
    	}
    }
     path+=".html";    
    java.net.URL helpURL = MenuListener.class.getResource(path);
    if (helpURL != null) {
        try {
            editorPane.setPage(helpURL);
        } catch (IOException e1) {
            System.err.println("Attempted to read a bad URL: " + helpURL);
        }
    } else {
        System.err.println("Couldn't find file: "+path);
    }

//    Put the editor pane in a scroll pane.
    JScrollPane editorScrollPane = new JScrollPane(editorPane);
    editorScrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    editorScrollPane.setPreferredSize(new Dimension(250, 145));
    editorScrollPane.setMinimumSize(new Dimension(10, 10));
    frame.getContentPane().add(editorScrollPane);
    frame.setVisible(true);
  }  
  else if (MenuListener.TOOLS_START_FILE.equals(cmd)){   // dfinir les fichiers de dmarrage
  	cadre.demOpen();
  }
  else if (MenuListener.TOOLS_ERASER.equals(cmd)){	// procedure eraser
  	cadre.eraserOpen();
  }
  }
}
